#!/usr/bin/env python
#
# Copyright (c) 2024, Marvell.
#
# SPDX-License-Identifier: MIT
#
# These scripts can be used for parsing and verifying LiquidSecurity HSM
# adapter attestation data; you may redistribute it and/or modify it under
# the terms of the MIT license.
#
from OpenSSL import crypto
import gzip, sys, pdb, io, hashlib
from cryptography import x509
from cryptography.hazmat.backends import default_backend

buffer = io.StringIO()

_END_DELIM = "-----END CERTIFICATE-----"

def verify_3_x(blob, signature, certs_file):
    """Verifies a message against a signature for firmware version 3.x.

      Args:
        blob: string, The message to verify.
        signature: string, The signature that is used for the verification.
        certs_file: certificate from which we will extract the Public Key

      Returns:
        True if message was signed by the private key associated with the public
        key that this object was constructed with.
        else returns False
    """
    with open(certs_file, 'r') as cert_data:
        certs = [cert + _END_DELIM for cert in cert_data.read().split(_END_DELIM) if cert.strip('\t\n\r')]
        for cert in certs:
            cert_obj = crypto.load_certificate(crypto.FILETYPE_PEM, cert)
            try:
                crypto.verify(cert_obj, signature, blob, 'sha256')
                print("RSA signature with PKCS1 padding verified")
                return True
            except crypto.Error as er:
                continue
        return False

def verify_2_x(blob, signature, certs_file):
    """Verifies a message against a signature for firmware version 2.x.

      Args:
        blob: bytes, The message to verify.
        signature: bytes, The signature that is used for the verification.
        certs_file: certificate from which we will extract the Public Key

      Returns:
        True if message was signed by the private key associated with the public
        key that this object was constructed with.
        else returns False
    """
    with open(certs_file, 'rb') as cert_file:
        cert_data = cert_file.read()
        certs = cert_data.split(_END_DELIM.encode() + b'\n')
        certs = [cert + _END_DELIM.encode() for cert in certs if cert]
        for cert in certs:
            try:
                # Load the certificate
                current_cert = x509.load_pem_x509_certificate(cert, default_backend())
                public_key = current_cert.public_key()
                
                # Decrypt the signature using the public key
                sig_int = int.from_bytes(signature, byteorder='big')
                decrypted_signature = pow(sig_int, public_key.public_numbers().e, public_key.public_numbers().n)
                decrypted_signature_bytes = decrypted_signature.to_bytes((decrypted_signature.bit_length() + 7) // 8, byteorder='big')
                
                # Verify the decrypted signature
                hash_value = hashlib.sha256(blob).digest()
                if decrypted_signature_bytes[-len(hash_value):] == hash_value:
                    print("RSA signature with raw padding verified")
                    return True
            except Exception as e:
                buffer.write(f"\nFailed with error {e}\n")
                continue
        return False

print('*************************************************************************')
print('Usage: ./verify_attest.py <partition.cert> <attestation.dat>')
print('*************************************************************************')
print('')

attest_data = open(sys.argv.pop(), 'rb').read()
certs_file = sys.argv.pop()

blob, signature = attest_data[:-256], attest_data[-256:]

# Attempt to verify using verify_3_x first, then fallback to verify_2_x
verified = verify_3_x(blob, signature, certs_file)
if not verified:
    verified = verify_2_x(blob, signature, certs_file)

print(f"Signature verification passed!" if verified else f"Signature verification failed! {buffer.getvalue()}")